      PROGRAM EX0506
      PARAMETER (IS=2)
      DIMENSION X(IS),TC(IS),PC(IS),DC(IS),OMEG(IS)
      DIMENSION Y1(IS),Y2(IS),Y3(IS),Y4(IS),BK(IS,IS),FUG(IS)
      CHARACTER*4 EOS(3)
      DATA EOS/'RKS','P&R','BWR'/
      DATA X(1),TC(1),PC(1),DC(1),OMEG(1)/0.324,568.95,2.49E+6,2032.5,
     &0.398/
      DATA X(2),TC(2),PC(2),DC(2),OMEG(2)/0.676,562.06,4.895E+6,3861.0,
     &0.212/
      DATA Y1(1),Y2(1),Y3(1),Y4(1)/-6.096,0.7712,-4.195E-4,8.855E-8/
      DATA Y1(2),Y2(2),Y3(2),Y4(2)/-33.92,0.4739,-3.017E-4,7.13E-8/
      DATA BK(1,1),BK(2,2)/0.,0./
      DATA T,P,IP/470.,1.4E+6,1/
C      DATA T,P,IP/590.,9.7E+6,1/
      WMOL=X(1)*0.1142302+X(2)*0.0781134
      WRITE (*,1000) T,P*1.E-6
 1000 FORMAT(5X,'@ ',F6.2,' K  & ',F6.2,' MPA',/,
     &5X,'  EOS    DENSITY  ENTHALPY  ENTROPY  PART.FUG.1  PART.FUG.2',
     &'  FUG.MIX.'/,5X,'         (KG/M3)  (KJ/KG)  (KJ/KG/K)    (-)'
     &,'         (-)        (-)')
C     RKS
      BK(1,2)=0.
      BK(2,1)=0.
      CALL RKS(T,P,IP,IS,X,TC,PC,OMEG,Y1,Y2,Y3,Y4,BK,D,H,S,FUG)
      FUGMIX=EXP(X(1)*LOG(FUG(1))+X(2)*LOG(FUG(2)))
      WRITE (*,1200) EOS(1),D*WMOL,H/WMOL/1000.,S/WMOL/1000.,FUG(1),
     &FUG(2),FUGMIX
C     PR
      BK(1,2)=0.
      BK(2,1)=0.
      CALL PR (T,P,IP,IS,X,TC,PC,OMEG,Y1,Y2,Y3,Y4,BK,D,H,S,FUG)
      FUGMIX=EXP(X(1)*LOG(FUG(1))+X(2)*LOG(FUG(2)))
      WRITE (*,1200) EOS(2),D*WMOL,H/WMOL/1000.,S/WMOL/1000.,FUG(1),
     &FUG(2),FUGMIX
C     BWR
      BK(1,2)=0.
      BK(2,1)=0.
      CALL BWR(T,P,IP,IS,X,TC,DC,OMEG,Y1,Y2,Y3,Y4,BK,D,H,S,FUG)
      FUGMIX=EXP(X(1)*LOG(FUG(1))+X(2)*LOG(FUG(2)))
      WRITE (*,1200) EOS(3),D*WMOL,H/WMOL/1000.,S/WMOL/1000.,FUG(1),
     &FUG(2),FUGMIX
 1200 FORMAT(5X,A6,F9.1,F9.1,F10.2,F11.4,F12.4,F11.4)
      END



      SUBROUTINE RKS
     &(T,P,IP,IC,X,TC,PC,OMEG,Y1,Y2,Y3,Y4,BK,D,H,S,FUG)
C
C     The subroutine calculates the following thermodynamic properties
C     of mixtures, as a function of temperature, T in [K], and pressure,
C     P in [PA]:
C          1) Molar Density                     (D)  in [MOL/M3]
C          2) Molar Enthalpy                    (H)  in [J/MOL]
C          3) Molar Entropy                     (S)  in [J/MOL/K]
C          4) Partial Fugacity Coefficient    (FUG)  in [-]
C     based on the Soave EoS, shown in Table 6.5
C
C     - Enthalpy and Entropy are arbitrary assumed to be equal to zero
C       at 273.15 K and 0.1013 MPA.
C     - Perfect-gas properties are calculated from perfect-gas Heat
C       Capacities as: CP=W1+W2*T+W3*(T**2)+W4*(T**3) in [J/MOL/K]
C       where W1=Sum(Xi*Y1i) etc.
C     - Two Switches are required from the subroutine:
C       IP = Phase specification switch (Liquid=1, Vapour=2)
C       IC = Number of components in mixture (Pure fluid=1)
C     - Other data required for each component (I) are:
C       mole fraction X (-), critical temperature, TC in [K], critical
C       pressure in [PA], and acentric factor, OMEG.
C     - Binary interaction parameters BK(I,J) must also be specified.
C
      DIMENSION X(IC),TC(IC),PC(IC),OMEG(IC),Y1(IC),Y2(IC),Y3(IC),Y4(IC)
      DIMENSION BK(IC,IC),FUG(IC)
      DIMENSION A(10),B(10),FM(10),ALP(10),AS(10)
      DATA R,T0,P0/8.314,273.15,0.101325E+6/
C
C     Parameters calculation AT,BT,CT,DT &  AST,ALP
C
      BT=0.
      DO 100 I=1,IC
      AS(I)=0.42747*R*R*TC(I)*TC(I)/PC(I)
      FM(I)=0.48508+1.55171*OMEG(I)-0.15613*(OMEG(I)**2)
      ALP(I)=(1.+FM(I)*(1.-(T/TC(I))**.5))**2
      A(I)=0.42747*ALP(I)*(P/PC(I))/((T/TC(I))**2)
      B(I)=0.08664*(P/PC(I))/(T/TC(I))
  100 BT=BT+X(I)*B(I)
      AAT=0.
      AT=0.
      DT=0.
      DO 200 I=1,IC
      DO 200 J=1,IC
      BIP=(1.-BK(I,J))
      AAT=AAT+X(I)*X(J)*BIP*((AS(I)*AS(J)*ALP(I)*ALP(J))**.5)
      AT=AT+X(I)*X(J)*BIP*((A(I)*A(J))**.5)
  200 DT=DT+X(I)*X(J)*FM(J)*BIP*((AS(I)*ALP(I)*AS(J)*(T/TC(J)))**.5)
C
C     Density Calculation
C
      IF (IP.EQ.1) Z=0.01
      IF (IP.EQ.2) Z=0.8
  300 F=(Z**3)-Z*Z+(AT-BT**2-BT)*Z-(AT*BT)
      F1=(3.*Z*Z)-(2.*Z)+(AT-(BT**2)-BT)
      H=F/F1
      Z=Z-H
      IF (ABS(H/Z).LE.0.0001) GOTO 400
      GOTO 300
  400 D=P/(Z*R*T)
C
C     Perfect-gas Enthalpy & Entropy
C
      W1=0.
      W2=0.
      W3=0.
      W4=0.
      DO 500 I=1,IC
      W1=W1+X(I)*Y1(I)
      W2=W2+X(I)*Y2(I)
      W3=W3+X(I)*Y3(I)
  500 W4=W4+X(I)*Y4(I)
      PGH=W1*(T-T0)+W2*((T**2)-(T0**2))/2.+W3*((T**3)-(T0**3))/3.
     &+W4*((T**4)-(T0**4))/4.
      PGS=W1*LOG(T/T0)+W2*(T-T0)+W3*((T**2)-(T0**2))/2.
     &+W4*((T**3)-(T0**3))/3.-R*LOG(P/P0)
C
C     Molar Enthalpy & Entropy
C
      DH=-1.+Z-(AT/BT)*(1.+(DT/AAT))*LOG(1.+(BT*D*R*T/P))
      H=(PGH+R*T*DH)
      DS=LOG(Z-BT)-(AT*DT/(BT*AAT))*LOG(1.+(BT*D*R*T/P))
      S=PGS+(R*DS)
C
C     Partial Fugacities
C
      DO 700 I=1,IC
      XAAT=0.
      DO 800 J=1,IC
  800 XAAT=XAAT+X(J)*(1.-BK(I,J))*((AS(I)*AS(J)*ALP(I)*ALP(J))**.5)
      FILN=(B(I)*(Z-1.)/BT)-LOG(Z-BT)+ (AT/BT)*((B(I)/BT)-(2.*XAAT/AAT))
     &*LOG(1.+(BT*D*R*T/P))
  700 FUG(I)=EXP(FILN)
      RETURN
      END

      SUBROUTINE PR
     &(T,P,IP,IC,X,TC,PC,OMEG,Y1,Y2,Y3,Y4,BK,D,H,S,FUG)
C
C     The subroutine calculates the following thermodynamic properties
C     of mixtures, as a function of temperature, T in [K], and pressure,
C     P in [PA]:
C          1) Molar Density                     (D)  in [MOL/M3]
C          2) Molar Enthalpy                    (H)  in [J/MOL]
C          3) Molar Entropy                     (S)  in [J/MOL/K]
C          4) Partial Fugacity Coefficient    (FUG)  in [-]
C     based on the Peng & Robinson EoS, shown in Table 6.6
C
C     - Enthalpy and Entropy are arbitrary assumed to be equal to zero
C       at 273.15 K and 0.1013 MPA.
C     - Perfect-gas properties are calculated from perfect-gas Heat
C       Capacities as: CP=W1+W2*T+W3*(T**2)+W4*(T**3) in [J/MOL/K]
C       where W1=Sum(Xi*Y1i) etc.
C     - Two Switches are required from the subroutine:
C       IP = Phase specification switch (Liquid=1, Vapour=2)
C       IC = Number of components in mixture (Pure fluid=1)
C     - Other data required for each component (I) are:
C       mole fraction X (-), critical temperature, TC in [K], critical
C       pressure in [PA], and acentric factor, OMEG.
C     - Binary interaction parameters BK(I,J) must also be specified.
C
      DIMENSION X(IC),TC(IC),PC(IC),OMEG(IC),Y1(IC),Y2(IC),Y3(IC),Y4(IC)
      DIMENSION BK(IC,IC),FUG(IC)
      DIMENSION A(10),B(10),FM(10),ALP(10),AS(10)
      DATA R,T0,P0/8.314,273.15,0.101325E+6/
C
C     Parameters calculation AT,BT,CT,DT &  AST,ALP
C
      BT=0.
      DO 100 I=1,IC
      AS(I)=0.45724*R*R*TC(I)*TC(I)/PC(I)
      FM(I)=0.37464+1.54226*OMEG(I)-0.26992*(OMEG(I)**2)
      ALP(I)=(1.+FM(I)*(1.-(T/TC(I))**.5))**2
      A(I)=0.45724*ALP(I)*(P/PC(I))/((T/TC(I))**2)
      B(I)=0.0778*(P/PC(I))/(T/TC(I))
  100 BT=BT+X(I)*B(I)
      AAT=0.
      AT=0.
      DT=0.
      DO 200 I=1,IC
      DO 200 J=1,IC
      BIP=(1.-BK(I,J))
      AAT=AAT+X(I)*X(J)*BIP*((AS(I)*AS(J)*ALP(I)*ALP(J))**.5)
      AT=AT+X(I)*X(J)*BIP*((A(I)*A(J))**.5)
  200 DT=DT+X(I)*X(J)*FM(J)*BIP*((AS(I)*ALP(I)*AS(J)*(T/TC(J)))**.5)
C
C     Density Calculation
C
      IF (IP.EQ.1) Z=0.01
      IF (IP.EQ.2) Z=0.8
  300 F=(Z**3)-(1.-BT)*Z*Z+(AT-3.*(BT**2)-2.*BT)*Z-(AT*BT-BT*BT-BT**3)
      F1=(3.*Z*Z)-(2.*Z*(1.-BT))+(AT-3.*(BT**2)-2.*BT)
      H=F/F1
      Z=Z-H
      IF (ABS(H/Z).LE.0.0001) GOTO 400
      GOTO 300
  400 D=P/(Z*R*T)
C
C     Perfect-gas Enthalpy & Entropy
C
      W1=0.
      W2=0.
      W3=0.
      W4=0.
      DO 500 I=1,IC
      W1=W1+X(I)*Y1(I)
      W2=W2+X(I)*Y2(I)
      W3=W3+X(I)*Y3(I)
  500 W4=W4+X(I)*Y4(I)
      PGH=W1*(T-T0)+W2*((T**2)-(T0**2))/2.+W3*((T**3)-(T0**3))/3.
     &+W4*((T**4)-(T0**4))/4.
      PGS=W1*LOG(T/T0)+W2*(T-T0)+W3*((T**2)-(T0**2))/2.
     &+W4*((T**3)-(T0**3))/3.-R*LOG(P/P0)
C
C     Molar Enthalpy & Entropy
C
      DH=1.-Z+(AT/(2.828*BT))*(1.+DT/AAT)*LOG((Z+2.414*BT)/(Z-0.414*BT))
      H=(PGH-R*T*DH)
      DS=LOG(Z-BT)-(AT*DT/(2.828*BT*AAT))*LOG((Z+2.414*BT)/
     &(Z-0.414*BT))
      S=PGS+(R*DS)
C
C     Partial Fugacities
C
      DO 700 I=1,IC
      XAAT=0.
      DO 800 J=1,IC
      BIP=(1.-BK(I,J))
  800 XAAT=XAAT+X(J)*BIP*((AS(I)*AS(J)*ALP(I)*ALP(J))**.5)
      FILN=(B(I)*(Z-1.)/BT)-LOG(Z-BT)+ (AT/(2.828*BT))*((B(I)/BT)-
     &(2.*XAAT/AAT))*LOG((Z+2.414*BT)/(Z-0.414*BT))
  700 FUG(I)=EXP(FILN)
      RETURN
      END

      SUBROUTINE BWR
     &(T,P,IP,IC,X,TC,DC,OMEG,Y1,Y2,Y3,Y4,BK,D,H,S,FUG)
C
C     The subroutine calculates the following thermodynamic properties
C     of mixtures, as a function of temperature, T in [K], and pressure,
C     P in [PA]:
C          1) Molar Density                     (D)  in [MOL/M3]
C          2) Molar Enthalpy                    (H)  in [J/MOL]
C          3) Molar Entropy                     (S)  in [J/MOL/K]
C          4) Partial Fugacity Coefficient    (FUG)  in [-]
C     based on the B-W-R Han-Starling EoS, shown in Table 6.7
C
C     - Enthalpy and Entropy are arbitrary assumed to be equal to zero
C       at 273.15 K and 0.1013 MPA.
C     - Perfect-gas properties are calculated from perfect-gas Heat
C       Capacities as: CP=W1+W2*T+W3*(T**2)+W4*(T**3) in [J/MOL/K]
C       where W1=Sum(Xi*Y1i) etc.
C     - Two Switches are required from the subroutine:
C       IP = Phase specification switch (Liquid=1, Vapour=2)
C       IC = Number of components in mixture (Pure fluid=1)
C     - Other data required for each component (I) are:
C       mole fraction X (-), critical temperature, TC in [K], critical
C       density in [MOL/M3], and acentric factor, OMEG.
C     - Binary interaction parameters BK(I,J) must also be specified.
C
      DIMENSION X(IC),TC(IC),DC(IC),OMEG(IC),Y1(IC),Y2(IC),Y3(IC),Y4(IC)
      DIMENSION BK(IC,IC),FUG(IC)
      DIMENSION A0(10),B0(10),C0(10),D0(10),E0(10),AS(10),BS(10),CS(10),
     &DS(10),ALP(10),GAM(10),A(11),B(11)
      DATA A/0.443690,1.28438,0.356306,0.544979,0.528629,0.484011,
     &0.0705233,0.504087,0.0307452,0.0732828,0.006450/
      DATA B/0.115449,-0.920731,1.70871,-0.270896,0.349261,0.754130,
     &-0.044448,1.32245,0.179433,0.463492,-0.022143/
      DATA R,T0,P0/8.314,273.15,0.101325E+6/
C
C     Parameters A0(I),B0(I),C0(I),D0(I),E0(I),
C                AS(I),BS(I),CS(I),DS(I),     &    ALP(I),GAM(I)
C
      DO 100 I=1,IC
      A0(I)=(R*TC(I)/DC(I))*(A(2)+B(2)*OMEG(I))
      B0(I)=(1./DC(I))*(A(1)+B(1)*OMEG(I))
      C0(I)=(R*(TC(I)**3)/DC(I))*(A(3)+B(3)*OMEG(I))
      D0(I)=(R*(TC(I)**4)/DC(I))*(A(9)+B(9)*OMEG(I))
      E0(I)=(R*(TC(I)**5)/DC(I))*(A(11)+B(11)*OMEG(I)*EXP(-3.8*OMEG(I)))
      AS(I)=(R*TC(I)/(DC(I)**2))*(A(6)+B(6)*OMEG(I))
      BS(I)=(1./(DC(I)**2))*(A(5)+B(5)*OMEG(I))
      CS(I)=(R*(TC(I)**3)/(DC(I)**2))*(A(8)+B(8)*OMEG(I))
      DS(I)=(R*(TC(I)**2)/(DC(I)**2))*(A(10)+B(10)*OMEG(I))
      ALP(I)=(1./(DC(I)**3))*(A(7)+B(7)*OMEG(I))
  100 GAM(I)=(1./(DC(I)**2))*(A(4)+B(4)*OMEG(I))
C
C     Total parameters A0T,B0T,C0T,D0T,E0T,
C                      AST,BST,CST,DST,    &  ALPT,GAMT  &  WMOLT
C
      A0T=0.
      B0T=0.
      C0T=0.
      D0T=0.
      E0T=0.
      AST=0.
      BST=0.
      CST=0.
      DST=0.
      ALPT=0.
      GAMT=0.
      WMOLT=0.
      DO 200 I=1,IC
      B0T=B0T+X(I)*B0(I)
      AST=AST+X(I)*(AS(I)**(1./3.))
      BST=BST+X(I)*(BS(I)**(1./3.))
      CST=CST+X(I)*(CS(I)**(1./3.))
      DST=DST+X(I)*(DS(I)**(1./3.))
      ALPT=ALPT+X(I)*(ALP(I)**(1./3.))
      GAMT=GAMT+X(I)*(GAM(I)**(1./2.))
      DO 200 J=1,IC
      BIP=(1.-BK(I,J))
      A0T=A0T+X(I)*X(J)*BIP*((A0(I)*A0(J))**0.5)
      C0T=C0T+X(I)*X(J)*(BIP**3.)*((C0(I)*C0(J))**0.5)
      D0T=D0T+X(I)*X(J)*(BIP**4.)*((D0(I)*D0(J))**0.5)
      E0T=E0T+X(I)*X(J)*(BIP**5.)*((E0(I)*E0(J))**0.5)
  200 CONTINUE
      AST=AST**3
      BST=BST**3
      CST=CST**3
      DST=DST**3
      ALPT=ALPT**3
      GAMT=GAMT**2
C
C     Density calculation
C
      IF (IP.EQ.1) D=100000.
      IF (IP.EQ.2) D=10.
  300 F=-P+(D*R*T)
      F=F+(B0T*R*T-A0T-(C0T/(T**2))+(D0T/(T**3))-(E0T/(T**4)))*D*D
      F=F+(BST*R*T-AST-(DST/T))*(D**3)+ALPT*(AST+(DST/T))*(D**6)
      F=F+(CST/(T**2))*(D**3)*(1.+GAMT*D*D)*EXP(-GAMT*D*D)
      DF=(R*T)+2.*D*(B0T*R*T-A0T-(C0T/(T**2))+(D0T/(T**3))-(E0T/(T**4)))
      DF=DF+3.*D*D*(BST*R*T-AST-(DST/T))
      DF=DF+6.*(D**5)*ALPT*(AST+(DST/T))
      DF=DF+(CST/(T**2))*3.*(D**2)*(1.+GAMT*D*D)*EXP(-GAMT*D*D)
      DF=DF+(CST/(T**2))*(D**3)*(2.*GAMT*D)*EXP(-GAMT*D*D)
      DF=DF-2.*GAMT*D*(CST/(T**2))*(D**3)*(1.+GAMT*D*D)*EXP(-GAMT*D*D)
      H=F/DF
      IF (ABS(H/D).LE.0.000001) GOTO 400
      D=D-H
      GOTO 300
C
C     Perfect-gas Enthalpy & Entropy
C
  400 W1=0.
      W2=0.
      W3=0.
      W4=0.
      DO 500 I=1,IC
      W1=W1+X(I)*Y1(I)
      W2=W2+X(I)*Y2(I)
      W3=W3+X(I)*Y3(I)
  500 W4=W4+X(I)*Y4(I)
      PGH=W1*(T-T0)+W2*((T**2)-(T0**2))/2.+W3*((T**3)-(T0**3))/3.
     &+W4*((T**4)-(T0**4))/4.
      PGS=W1*LOG(T/T0)+W2*(T-T0)+W3*((T**2)-(T0**2))/2.
     &+W4*((T**3)-(T0**3))/3.-R*LOG(P/P0)
C
C     Molar Enthalpy & Entropy
C
      GDD=GAMT*D*D
      H=PGH+D*((B0T*R*T)-(2.*A0T)-(4.*C0T/(T*T))+(5.*D0T/(T**3))-
     &(6.*E0T/(T**4))) + 0.5*(D**2)*(2.*BST*R*T-(3.*AST)-(4.*DST/T))
      H=H + 0.2*(D**5)*ALPT*((6.*AST)+(7.*DST/T))
      H=H + (CST/(GAMT*T*T))*(3.-(3.+0.5*GDD-GDD*GDD)*EXP(-GDD))
      S=PGS+R*LOG(P/(R*T*D))
      S=S - D*((B0T*R)+(2.*C0T/(T**3))-(3.*D0T/(T**4))+(4.*E0T/(T**5)))
      S=S - 0.5*(D**2)*((BST*R)+(DST/(T**2)))
      S=S + 0.2*(D**5)*ALPT*DST/(T**2)
      S=S + (2.*CST/(GAMT*(T**3)))*(1.-(1.+0.5*GDD)*EXP(-GDD))
C
C     Partial Fugacities
C
      DO 800 I=1,IC
      FU=(R*T*LOG(D*R*T))+(D*R*T*(B0T+B0(I)))
      FUS=0.
      DO 700 J=1,IC
      BIP=(1.-BK(I,J))
  700 FUS=FUS+X(J)*(-BIP*((A0(J)*A0(I))**0.5)
     &-(BIP**3.)*(((C0(J)*C0(I))**0.5)/(T*T))
     &+(BIP**4.)*(((D0(J)*D0(I))**0.5)/(T**3))
     &-(BIP**5.)*(((E0(J)*E0(I))**0.5)/(T**4)))
      AAI=(AST*AST*AS(I))**(1./3.)
      BBI=(BST*BST*BS(I))**(1./3.)
      CCI=(CST*CST*CS(I))**(1./3.)
      DDI=(DST*DST*DS(I))**(1./3.)
      ALPI=(ALPT*ALPT*ALP(I))**(1./3.)
      GDD=GAMT*D*D
      FU=FU+(2.*D*FUS)+(3.*D*D/2.)*((BBI*R*T)-AAI-(DDI/T))
      FU=FU+(3.*ALPT*(D**5)/5.)*(AAI+(DDI/T))
      FU=FU+(3.*(D**5)/5.)*(AST+(DST/T))*ALPI
      FU=FU+(3.*CCI*D*D/(T**2))*(((1.-EXP(-GDD))/GDD)-(0.5*EXP(-GDD)))
      FU=FU-(2.*CST/(GAMT*T*T))*((GAM(I)/GAMT)**.5)*
     &(1.-(1.+GDD+0.5*GDD*GDD)*EXP(-GDD))
  800 FUG(I)=(1./P)*EXP(FU/(R*T))
      RETURN
      END
