      PROGRAM EX0606
      PARAMETER (IC=2)
      DIMENSION X(IC),TC(IC),PC(IC),OMEG(IC),PHI(IC),PK(IC,IC)
      DATA T/323.15/
      DATA X(1),TC(1),PC(1),OMEG(1)/0.5,535.6,4.15E+6,0.329/
      DATA X(2),TC(2),PC(2),OMEG(2)/0.5,563.0,4.11E+6,0.263/
      DATA PK(1,1),PK(1,2),PK(2,1),PK(2,2)/1.,1.,1.,1./
      DATA H0RTCM,PHIMIX,PHIMIX1,ZMIX/-0.02835,0.9864,0.98673,0.9862/
      HTA=1.

      CALL NPFUGI
     &(IC,T,X,TC,PC,OMEG,H0RTCM,PK,HTA,PHIMIX,PHIMIX1,ZMIX,PHI)

      WRITE (*,1000) PHI(1),PHI(2)
 1000 FORMAT(1X,'FUGACITY COEFFICIENTS IN MIXTURE OF ',
     &/,3X,'COMPONENT 1 : ',F5.3,/,3X,'COMPONENT 2 : ',F5.3)
      END

      SUBROUTINE NPFUGI
     &(IC,T,X,TC,PC,OMEG,H0RTCM,PK,HTA,PHIMIX,PHIMIX1,ZMIX,PHI)
C
C     The Subroutine calculates the partial fugacity coefficient (PHI)
C     of component i in a mixture, from the mixture fugacity. The
C     Plocker mixing rules (Table 5.3) are employed.
C     Data required are:
C     - Temperature, T in [K], No of components, IC,
C     - For each component:  mole fractions, X in [-], critical
C       temperature, TC in [K], critical pressure, PC in [Pa],
C       acentric Factor, OMEG in [-], and
C     - For the mixture: residual enthalpy term, H0RTCM in [-],
C       fugacity, PHIMIX in [-], 1st correction to fugacity, PHIMIX1 in
C       [-] and compression factor, Zmix in [-].
C     - Plocker's HTA parameter and binary interaction parameters
C       PK(I,J) must also be specified.
C     Mixture critical parameters are calculated in the beginning.
C
      DIMENSION X(IC),TC(IC),PC(IC),OMEG(IC),PHI(IC),PK(IC,IC),VC(IC)
      R=8.31451
C
C     Mixture Acentric Factor (AFMIX)
C
      AFMIX=0.
      DO 100 I=1,IC
  100 AFMIX=AFMIX+X(I)*OMEG(I)
C
C     Component Critical Volume (VC(I)) & Mixture's (VCMIX)
C
      DO 120 I=1,IC
  120 VC(I)=(0.2905-0.085*OMEG(I))*R*TC(I)/PC(I)
      VVC=0.0
      DO 140 J=1,IC
      DO 140 K=1,IC
      VV=((VC(J)**(1./3.))+(VC(K)**(1./3.)))**3
  140 VVC=VVC+X(J)*X(K)*VV
      VCMIX=VVC/8.
C
C     Mixture Critical Parameters (TCMIX,ZCMIX,PCMIX)
C
      TCC=0.0
      DO 160 J=1,IC
      DO 160 K=1,IC
      VV=((VC(J)**(1./3.))+(VC(K)**(1./3.)))**(3.*HTA)
  160 TCC=TCC+X(J)*X(K)*VV*PK(J,K)*((TC(J)*TC(K))**0.5)
      TCMIX=TCC/((8.*VCMIX)**HTA)
      ZCMIX=(0.2905-0.085*AFMIX)
      PCMIX=ZCMIX*R*TCMIX/VCMIX
C
C     Calculation of Derivatives
C
      DO 600 I=1,IC
      DPHI=0.
      DO 500 K=1,IC
      IF (K.EQ.I) GO TO 500
      SUMVC=0.
      SUMTC=0.
      DO 400 L=1,IC
      TCLK=PK(L,K)*((TC(L)*TC(K))**0.5)
      TCLI=PK(L,I)*((TC(L)*TC(I))**0.5)
      VCLK=(1./8.)*((VC(L)**(1./3.)+VC(K)**(1./3.))**3.)
      VCLI=(1./8.)*((VC(L)**(1./3.)+VC(I)**(1./3.))**3.)
      SUMVC=SUMVC+X(L)*(VCLK-VCLI)
  400 SUMTC =SUMTC+X(L)*((VCLK**HTA)*TCLK-(VCLI**HTA)*TCLI)
      DVCDX=2.*SUMVC
      DTCDX=(2.*SUMTC-HTA*(VCMIX**(HTA-1.))*DVCDX*TCMIX)/(VCMIX**HTA)
      DOMDX=OMEG(K)-OMEG(I)
      DZCDX=-0.085 *DOMDX
      DPCDX=PCMIX*((DZCDX /ZCMIX)+(DTCDX/TCMIX)-(DVCDX/VCMIX)) 

      DLNFDX=(H0RTCM*DTCDX/T)+(DPCDX*(1.-ZMIX)/PCMIX)
     &+(DOMDX*LOG(PHIMIX1))
      DPHI=DPHI+X(K)*DLNFDX
  500 CONTINUE
  600 PHI(I)=EXP(LOG(PHIMIX)-DPHI)
      RETURN
      END
