      PROGRAM EX0709
      PARAMETER (IC=2)
      DIMENSION U(IC,IC),X(IC),R(IC),Q(IC),QPR(IC),G(IC)
      DATA U(1,1),U(1,2)/   0. ,-22.83/
      DATA U(2,1),U(2,2)/266.31,  0.  /
      DATA R/  2.57,3.82/
      DATA Q/  2.34,3.31/
      DATA QPR/2.34,3.31/
      DATA X,T/0.047,0.953,307./
C
      CALL UNIQUAC(IC,T,X,R,Q,QPR,U,G)
      WRITE(*,1000) G(1),G(2)
 1000 FORMAT(3X,'G1 =',F6.4,5X,'G2 =',F6.4)
      STOP
      END


      SUBROUTINE UNIQUAC(IC,T,X,R,Q,QPR,U,G)
C
C     The subroutine calculates activity coefficients G(I) from the
C     UNIQUAC equations, for a mixture at temperature T [K], of IC
C     components each of molefraction X(I).
C     Other input required is
C     - R(I)   : r(i), volume parameter of component i
C     - Q(I)   : q(i), area parameter of component i
C     - QPR(I) : q'(i), area parameter for water and alcohols
C                       for all other compounds equal to q(i), and
C     - U(I,J) : u(I,J)/R
C
      DIMENSION U(IC,IC),X(IC),R(IC),Q(IC),QPR(IC),G(IC)
      DIMENSION PHI(IC),THI(IC),THIPR(IC),WL(IC),TA(IC,IC)
      SPHI=0.
      STHI=0.
      STHIPR=0.
      DO 100 J=1,IC
      SPHI=SPHI+X(J)*R(J)
      STHI=STHI+X(J)*Q(J)
  100 STHIPR=STHIPR+X(J)*QPR(J)
      DO 120 I=1,IC
      PHI(I)=X(I)*R(I)/SPHI
      THI(I)=X(I)*Q(I)/STHI
      THIPR(I)=X(I)*QPR(I)/STHIPR
  120 WL(I)=5.*(R(I)-Q(I))-(R(I)-1.)
      DO 140 I=1,IC
      DO 140 J=1,IC
  140 TA(I,J)=EXP(-U(I,J)/T)
C
C     Configurational part
C
      DO 220 I=1,IC
      SXL=0.
      STHTJI=0.
      DO 160 J=1,IC
      SXL=SXL+X(J)*WL(J)
  160 STHTJI=STHTJI+TA(J,I)*THIPR(J)
      GC=LOG(PHI(I)/X(I))+5.*Q(I)*LOG(THI(I)/PHI(I))
      GC=GC+WL(I)-PHI(I)*SXL/X(I)
C
C     Residual part
C
      FS=0.
      DO 200 J=1,IC
      STHT=0.
      DO 180 K=1,IC
  180 STHT=STHT+THIPR(K)*TA(K,J)
  200 FS=FS+THIPR(J)*TA(I,J)/STHT
      GR=QPR(I)*(1.-LOG(STHTJI)-FS)
C
C     Activity coefficient
C
  220 G(I)=EXP(GC+GR)
      RETURN
      END