      PROGRAM EX0802
      PARAMETER (IS=2)
      DIMENSION X(IS),Y(IS),TC(IS),PC(IS),OMEG(IS),FUGV(IS),FUGL(IS)
      DIMENSION BK(IS,IS)
      DATA (TC  (I),I=1,IS)/304.1,305.33/
      DATA (PC  (I),I=1,IS)/7.380e+6,4.871E+6/
      DATA (OMEG(I),I=1,IS)/0.239,0.099/
c      DATA BK(1,1),BK(1,2),BK(2,1),BK(2,2)/0.,0.124,0.124,0./
      DATA BK(1,1),BK(1,2),BK(2,1),BK(2,2)/0.,0.,0.,0./
      DATA ITP,P,WLO,WHI,X/1,3.025E+6,250.,280.,0.69,0.31/

      CALL BUBLPR(T,P,ITP,WLO,WHI,IS,X,Y,TC,PC,BK,OMEG,FUGL,FUGV)

   40 WRITE (*,1100) T,P*1.E-6
 1100 FORMAT(3X,/,4X,'>>>  T=',F7.2,'K  &  P=',F6.3,'MPA'/,4X,70('-'),/,
     &11X,'X(I)    Y(I)',10X,'FUG.LIQ.',6X,'FUG.VAP.',11X,'K')
      DO 60 J=1,IS
   60 WRITE (*,1200) J,X(J),Y(J),FUGL(J),FUGV(J),FUGL(J)/FUGV(J)
 1200 FORMAT(4X,'(',I1,')',F8.4,F9.4,2X,2(4X,F10.5),6X,F10.5)
      END


      SUBROUTINE BUBLPR(T,P,ITP,WLO,WHI,IS,X,Y,TC,PC,BK,OMEG,FUGL,FUGV)
      DIMENSION X(IS),Y(IS),TC(IS),PC(IS),OMEG(IS),FUGV(IS),FUGL(IS)
      DIMENSION BK(IS,IS)
C
C     The subroutine calculates vapour molefractions Y(I) of a mixture
C     of IS components, and
C     a) ITP=1, finds bubble-point temperature T in [K],
C               given the pressure P in [Pa], or
C     b) ITP=2, finds bubble-point pressure P in [MPa],
C               given the temperature T in [K].
C        Method adopted is based on the calculation of the equilibrium
C     constant from the liquid fugacity coefficients, FUGL(I), and the
C     vapour fugacity coefficient FUGV(I) from the Pend & Robinson EoS.
C     -  Additional input to the subroutine are liquid molefractions
C     X(I), critical constants TC(I) in [K], and PC(I) in [Pa], the
C     acentric factor OMEG(I), and the PR binary interaction parameters
C     BK(I,J).
C        Also first and last value, WLO and WHI, for the temperature or
C     pressure iteration must be provide.
C     -  Output includes the liquid and vapour fugacity coefficients.
C
C     INITIAL GUESSES FOR Y(I)
C
      S1=0.
      DO 100 I=1,IS
      Y(I)=X(I)
  100 S1=S1+Y(I)
C
   20 PROP=(WLO+WHI)/2.
      IF (ITP.EQ.1) T=PROP
      IF (ITP.EQ.2) P=PROP
      WRITE (*,*) PROP
   30 CALL PR(T,P,2,IS,Y,TC,PC,OMEG,Y1,Y2,Y3,Y4,BK,D,H,S,FUGV)
      CALL PR(T,P,1,IS,X,TC,PC,OMEG,Y1,Y2,Y3,Y4,BK,D,H,S,FUGL)
      CHKT=ABS(FUGV(1)-FUGL(1))*100./FUGV(1)
      IF (CHKT.GT.0.001) GOTO 40
      WRITE (*,*) '  NO CONVERGENCE - TRY CLOSER TEMPERATURE LIMITS'
      STOP
   40 SY=0.
      DO 120 I=1,IS
      Y(I)=X(I)*FUGL(I)/FUGV(I)
  120 SY=SY+Y(I)
      DO 140 I=1,IS
  140 Y(I)=Y(I)/SY
      IF (ABS(SY-S1).LE.0.000001) GOTO 50
      S1=SY
      GOTO 30
C
C     TEMPERATURE (ITP=1) OR PRESSURE (ITP=2) ITERATION
C
   50 IF (SY.GE.0.99995.AND.SY.LE.1.00005) GOTO 60
      IF (ITP.EQ.1.AND.SY.GT.1.00005) WHI=T
      IF (ITP.EQ.1.AND.SY.LT.0.99995) WLO=T
      IF (ITP.EQ.2.AND.SY.GT.1.00005) WLO=P
      IF (ITP.EQ.2.AND.SY.LT.0.99995) WHI=P
      GOTO 20
   60 RETURN
      END


      SUBROUTINE PR
     &(T,P,IP,IS,X,TC,PC,OMEG,Y1,Y2,Y3,Y4,BK,D,H,S,FUG)
C
C     The subroutine calculates the following thermodynamic properties
C     of mixtures, as a function of temperature, T in [K], and pressure,
C     P in [PA]:
C          1) Density                           (D)  in [MOL/M3]
C          2) Enthalpy                          (H)  in [J/MOL]
C          3) Entropy                           (S)  in [J/MOL/K]
C          4) Partial Fugacity Coefficient    (FUG)  in [-]
C     based on the Peng & Robinson EoS, shown in Table 6.6
C
C     - Enthalpy and Entropy are arbitrary assumed to be equal to zero
C       at 273.15 K and 0.1013 MPA.
C     - Perfect-gas properties are calculated from perfect-gas Heat
C       Capacities as: CP=W1+W2*T+W3*(T**2)+W4*(T**3) in [J/MOL/K]
C       where W1=Sum(Xi*Y1i) etc.
C     - Two Switches are required from the subroutine:
C       IP = Phase specification switch (Liquid=1, Vapour=2)
C       IS = Number of components in mixture (Pure fluid=1)
C     - Other data required for each component (I) are:
C       MoleFraction X (-), Critical Temperature, TC in [K], Critical
C       Pressure in [PA], and Acentric factor, OMEG.
C     - Binary interaction parameters BK(I,J) must also be specified.
C
      DIMENSION X(IS),TC(IS),PC(IS),OMEG(IS),Y1(IS),Y2(IS),Y3(IS),Y4(IS)
      DIMENSION BK(IS,IS),FUG(IS)
      DIMENSION A(10),B(10),FM(10),ALP(10),AS(10)
      DATA R,T0,P0/8.314,273.15,0.101325E+6/
C
C     Parameters calculation AT,BT,CT,DT &  AST,ALP
C
      BT=0.
      DO 100 I=1,IS
      AS(I)=0.45724*R*R*TC(I)*TC(I)/PC(I)
      FM(I)=0.37464+1.54226*OMEG(I)-0.26992*(OMEG(I)**2)
      ALP(I)=(1.+FM(I)*(1.-(T/TC(I))**.5))**2
      A(I)=0.45724*ALP(I)*(P/PC(I))/((T/TC(I))**2)
      B(I)=0.0778*(P/PC(I))/(T/TC(I))
  100 BT=BT+X(I)*B(I)
      AAT=0.
      AT=0.
      DT=0.
      DO 200 I=1,IS
      DO 200 J=1,IS
      BIP=(1.-BK(I,J))
      AAT=AAT+X(I)*X(J)*BIP*((AS(I)*AS(J)*ALP(I)*ALP(J))**.5)
      AT=AT+X(I)*X(J)*BIP*((A(I)*A(J))**.5)
  200 DT=DT+X(I)*X(J)*FM(J)*BIP*((AS(I)*ALP(I)*AS(J)*(T/TC(J)))**.5)
C
C     Density Calculation
C
      IF (IP.EQ.1) Z=0.01
      IF (IP.EQ.2) Z=0.8
  300 F=(Z**3)-(1.-BT)*Z*Z+(AT-3.*(BT**2)-2.*BT)*Z-(AT*BT-BT*BT-BT**3)
      F1=(3.*Z*Z)-(2.*Z*(1.-BT))+(AT-3.*(BT**2)-2.*BT)
      H=F/F1
      Z=Z-H
      IF (ABS(H/Z).LE.0.0001) GOTO 400
      GOTO 300
  400 D=P/(Z*R*T)
C
C     Perfect-gas Enthalpy & Entropy
C
      W1=0.
      W2=0.
      W3=0.
      W4=0.
      DO 500 I=1,IS
      W1=W1+X(I)*Y1(I)
      W2=W2+X(I)*Y2(I)
      W3=W3+X(I)*Y3(I)
  500 W4=W4+X(I)*Y4(I)
      PGH=W1*(T-T0)+W2*((T**2)-(T0**2))/2.+W3*((T**3)-(T0**3))/3.
     &+W4*((T**4)-(T0**4))/4.
      PGS=W1*LOG(T/T0)+W2*(T-T0)+W3*((T**2)-(T0**2))/2.
     &+W4*((T**3)-(T0**3))/3.-R*LOG(P/P0)
C
C     Enthalpy & Entropy
C
      DH=1.-Z+(AT/(2.828*BT))*(1.+DT/AAT)*LOG((Z+2.414*BT)/(Z-0.414*BT))
      H=(PGH-R*T*DH)
      DS=LOG(Z-BT)-(BT*DT/(2.828*AT*AAT))*LOG((Z+2.414*BT)/
     &(Z-0.414*BT))
      S=PGS+(R*DS)
C
C     Partial Fugacities
C
      DO 700 I=1,IS
      XAAT=0.
      DO 800 J=1,IS
      BIP=(1.-BK(I,J))
  800 XAAT=XAAT+X(J)*BIP*((AS(I)*AS(J)*ALP(I)*ALP(J))**.5)
      FILN=(B(I)*(Z-1.)/BT)-LOG(Z-BT)+ (AT/(2.828*BT))*((B(I)/BT)-
     &(2.*XAAT/AAT))*LOG((Z+2.414*BT)/(Z-0.414*BT))
  700 FUG(I)=EXP(FILN)
      RETURN
      END
