      PROGRAM EX0803
      PARAMETER (IS=2)
      DIMENSION X(IS),Y(IS),PSAT(IS),EK(IS),FPUR(IS),F(IS),G(IS)
      DIMENSION TC(IS,IS),PC(IS,IS),OMEG(IS,IS),VP(IS,3)
      DIMENSION SLR(IS,IS),V(IS,IS)
      DATA (TC  (J,J),J=1,IS)/536.74,647.3/
      DATA (PC  (J,J),J=1,IS)/5.168E+6,22.100E+6/
      DATA (OMEG(J,J),J=1,IS)/0.623,0.344/
      DATA (SLR (1,I),I=1,IS)/  0.  , 529.10/
      DATA (SLR (2,I),I=1,IS)/598.08,   0. /
      DATA (VP  (1,I),I=1,3)/6.87065,1438.587,-74.598/
      DATA (VP  (2,I),I=1,3)/7.06252,1650.270,-46.804/
      DATA (V   (1,I),I=1,IS)/1.    ,4.4184/
      DATA (V   (2,I),I=1,IS)/0.2307,1.    /
      DATA ITP,T,WLO,WHI,X/2,361.10,0.06E+6,0.15E+6,0.179,0.821/
C      DATA ITP,P,WLO,WHI,X/1,0.095E+6,350,370,0.179,0.821/

      CALL BUBLVW
     &(IS,P,T,ITP,WLO,WHI,X,Y,TC,PC,OMEG,SLR,V,PSAT,EK,FPUR,F,G,VP)

      WRITE (*,1100) T,P*1.E-6
 1100 FORMAT(3X,/,4X,'>>>  T=',F7.2,'K  &  P=',F6.3,'MPA'/,4X,70('-'),/,
     &11X,'X(I)    Y(I)     PSAT        K       FPUR        F        G')
      DO 20 J=1,IS
   20 WRITE (*,1200) J,X(J),Y(J),PSAT(J)/1.E+6,EK(J),FPUR(J),F(J),G(J)
 1200 FORMAT(4X,'(',I1,')',F8.4,F9.4,5F10.5)
      END


      SUBROUTINE BUBLVW
     &(IS,P,T,ITP,WLO,WHI,X,Y,TC,PC,OMEG,SLR,V,PSAT,EK,FPUR,F,G,VP)
C
C     The subroutine calculates vapour molefractions Y(I) of a mixture
C     of IS components, and
C     a) ITP=1, finds bubble-point temperature T in [K],
C               given the pressure P in [Pa], or
C     b) ITP=2, finds bubble-point pressure P in [MPa],
C               given the temperature T in [K].
C        Method adopted is based on:
C     1. Activity coeficients, G(I),(supplied by another subroutine
C        say TKWILSON), and
C     2. Virial equation truncated to second coefficient (Tsonopoulos
C        expressions with Lorentz-Berthelot mixing rules).
C     -  Addiotional input to the subroutine are liquid molefractions
C     X(I), critical constants TC(I,I) in [K], and PC(I,I) in [Pa]
C     acentric factor OMEG(I,I), and Antoine equation coeffs. VP(IS,3).
C        Also first and last value, WLO and WHI, for the temperature or
C     pressure iteration must be provide.
C     -  Output includes vapour pressure PSAT(I), equilibrium constant
C     EK(I),pure fugacity coef.FPUR(I) and component fugacity coef.F(I).
C
      DIMENSION X(IS),Y(IS),PSAT(IS),EK(IS),FPUR(IS),F(IS),G(IS)
      DIMENSION TC(IS,IS),PC(IS,IS),OMEG(IS,IS),VP(IS,3)
      DIMENSION SLR(IS,IS),V(IS,IS)
      DIMENSION B(10,10),VC(10,10),ZC(10,10)
      R=8.3145
C
C     Mixture Critical TC(I,J), PC(I,J) (Lorentz-Berthelot Mixing Rules)
C
      DO 100 I=1,IS
  100 VC(I,I)=(0.291-0.080*OMEG(I,I))*R*TC(I,I)/PC(I,I)
      DO 120 I=1,IS
      DO 120 J=1,IS
      OMEG(I,J)=(OMEG(I,I)+OMEG(J,J))/2.
      ZC(I,J)=0.291-0.080*OMEG(I,J)
      VC(I,J)=(1./8.)*((VC(I,I)**(1./3.))+(VC(J,J)**(1./3.)))**3
      TC(I,J)=SQRT(TC(I,I)*TC(J,J))
  120 PC(I,J)=ZC(I,J)*R*TC(I,J)/VC(I,J)
C
C     Activity Coef. G(I), Virial Coef. B(I,J) (Tsonopoulos Equations)
C
   10 PROP=(WLO+WHI)/2.
      IF (ITP.EQ.1) T=PROP
      IF (ITP.EQ.2) P=PROP
      WRITE(*,*) PROP
      CALL TKWILSON(IS,X,T,SLR,V,G,1)
      DO 140 I=1,IS
      DO 140 J=1,IS
      TRI=TC(I,J)/T
  140 B(I,J)=R*TC(I,J)/PC(I,J)*(0.1445-0.33*TRI-0.1385*(TRI**2)-0.0121*
     &(TRI**3)-0.000607*(TRI**8)+OMEG(I,J)*(0.0637+0.331*(TRI**2)-0.423*
     &(TRI**3)-0.008*(TRI**8)))
C
C     First Y(I)'s
C
      S1=0.
      DO 160 J=1,IS
      FPUR(J)=EXP(B(J,J)*P/R/T)
      PSAT(J)=1000.*(10.**(VP(J,1)-VP(J,2)/(T+VP(J,3))))
      Y(J)=G(J)*PSAT(J)*FPUR(J)/P
  160 S1=S1+Y(J)
      DO 180 J=1,IS
  180 Y(J)=Y(J)/S1
C
C     Pure Fugacity Coef. FPUR(I), Comp.Fugacity Coef. F(I),
C     Sat.Pressure PSAT(I), Equil.Constant EK(I), & Y(I)s
C
   20 BMIX=0.
      DO 200 I=1,IS
      DO 200 J=1,IS
  200 BMIX=BMIX+Y(I)*Y(J)*B(I,J)
      SY=0.
      DO 240 I=1,IS
      FTERM=0.
      DO 220 K=1,IS
  220 FTERM=FTERM+Y(K)*B(K,I)
      F(I)=EXP(P/R/T*(2.*FTERM-BMIX))
      FPUR(I)=EXP(B(I,I)*P/R/T)
      PSAT(I)=1000.*(10.**(VP(I,1)-VP(I,2)/(T+VP(I,3))))
      EK(I)=G(I)*FPUR(I)*PSAT(I)/(P*F(I))
      Y(I)=EK(I)*X(I)
  240 SY=SY+Y(I)
      DO 260 I=1,IS
  260 Y(I)=Y(I)/SY
      IF (ABS(SY-S1).LE.0.000001) GOTO 30
      S1=SY
      GOTO 20
C
C     TEMPERATURE (IPT=1) OR PRESSURE (IPT=2) ITERATION
C
   30 IF (SY.GE.0.99995.AND.SY.LE.1.00005) GOTO 40
      IF (ITP.EQ.1.AND.SY.GT.1.00005) WHI=T
      IF (ITP.EQ.1.AND.SY.LT.0.99995) WLO=T
      IF (ITP.EQ.2.AND.SY.GT.1.00005) WLO=P
      IF (ITP.EQ.2.AND.SY.LT.0.99995) WHI=P
      GOTO 10
   40 RETURN
      END


      SUBROUTINE TKWILSON(IS,X,T,SLR,V,G,IOPT)
C
C     The subroutine calculates activity coefficients G(I) for the
C     Wilson (IOPT=1) and the T-K-Wilson (IOPT=2) models, for a mixture
C     at temperature T [K], of IS components each of molefraction X(I).
C     Other input required is
C     - SLR(I,J) : lamda(I,J)/R and
C     -  V (I,J) : molar volume of J / molar volume of I.
C
      DIMENSION X(IS),SLR(IS,IS),V(IS,IS),G(IS)
      DIMENSION AL(5,5),VV(5,5)
      DO 100 I=1,IS
      DO 100 J=1,IS
      AL(I,J)=V(J,I)*EXP(-SLR(I,J)/T)
      IF (IOPT.EQ.1) VV(J,I)=1.
  100 IF (IOPT.EQ.2) VV(J,I)=V(J,I)
      DO 500 I=1,IS
      SXV=0.
      SXL=0.
      DO 200 J=1,IS
      SXV=SXV+X(J)*VV(I,J)
  200 SXL=SXL+X(J)*AL(I,J)
      G11=LOG(SXV/SXL)
      SXK=0.
      DO 400 K=1,IS
      SXV=0.
      SXL=0.
      DO 300 J=1,IS
      SXV=SXV+X(J)*VV(K,J)
  300 SXL=SXL+X(J)*AL(K,J)
  400 SXK=SXK+X(K)*((VV(K,I)/SXV)-(AL(K,I)/SXL))
  500 G(I)=EXP(G11+SXK)
      RETURN
      END
