      PROGRAM EX1003
      PARAMETER (IS=2)
      DIMENSION EK(IS),SIG(IS),WM(IS),VIS(IS),TCC(IS),Y(IS)
      DIMENSION Y1(IS),Y2(IS),Y3(IS),Y4(IS)
C     CO+Argon
      DATA EK(1),SIG(1),WM(1),Y(1)/ 91.7,0.3690,28.0104,0.592/
      DATA Y1(1),Y2(1),Y3(1),Y4(1)/30.87,-1.285E-2,2.789E-5,-1.272E-8/
      DATA EK(2),SIG(2),WM(2),Y(2)/ 93.3,0.3542,39.9480,0.408/
      DATA Y1(2),Y2(2),Y3(2),Y4(2)/20.8,0.,0.,0./
      DATA T/308.15/
      CALL TPDILG(IS,T,Y,EK,SIG,WM,Y1,Y2,Y3,Y4,VIS,VMIX,TCC,TMIX)
      WRITE(*,1000)
 1000 FORMAT(21X,'THERMAL CONDUCTIVITY      VISCOSITY',/,
     &27X,'(mW/M/K)',13X,'(uPa.s)')
      DO 20 I=1,IS
   20 WRITE(*,1100) I,TCC(I)*1.E+3,VIS(I)*1.E+6
 1100 FORMAT(5X,'COMPONENT  ',I2,9X,F6.2,15X,F6.2)
      IF (IS.GT.1) WRITE (*,1200) TMIX*1.E+3,VMIX*1.E+6
 1200 FORMAT(7X,'MIXTURE',13X,F6.2,15X,F6.2)
      END

      SUBROUTINE TPDILG(IS,T,Y,EK,SIG,WM,Y1,Y2,Y3,Y4,VIS,VMIX,TCC,TMIX)
C
C     The subroutine calculates the viscosity and thermal conductivity
C     of a mixture of IS gases at low pressures and temperature T (K).
C     Calculations are based on the following methods:
C     Viscosity
C     - Pure gases: Reduced collision integral (Neufeld)
C     - Mixtures  : Wilke expressions
C     Thermal Conductivity
C     - Pure gases: Modified Eucken correlation
C     - Mixtures  : Brokaw expressions
C     Data required for each component are: potential parameters e/k (K)
C     sigma (nm), EK,SIG, molecular weight WM, molefractions Y,
C     and perfect-gas heat capacity coefficients, Y1,Y2,Y3,Y4.
C
      DIMENSION EK(IS),SIG(IS),WM(IS),VIS(IS),TCC(IS),Y(IS)
      DIMENSION Y1(IS),Y2(IS),Y3(IS),Y4(IS)
      DIMENSION PHI(10,10),A(10,10),C(10,10),TCTR(10),TCINT(10)
      R=8.3145
C
C     Pure Components Viscosity VIS(I) & Thermal Conductivity TCC(I)
C
      DO 100 I=1,IS
      TS=T/EK(I)
      CP=Y1(I)+Y2(I)*T+Y3(I)*T**2+Y4(I)*T**3
      OMEGA=1.16145*(TS**(-0.14874))+0.52487*(EXP(-0.7732*TS))+2.16178*
     &(EXP(-2.43787*TS))-6.435E-4*(TS**0.14874)*SIN(18.0323*(TS**
     &(-0.7683))-7.2371)
      VIS(I)=26.69E-9*((WM(I)*T)**0.5)/(SIG(I)*SIG(I)*OMEGA)
  100 TCC(I)=(VIS(I)*(CP-R)/(WM(I)/1000.))*(1.32+1.77/((CP/R)-1))
C
C     Mixture Viscocity VMIX
C
      DO 120 I=1,IS
      DO 120 J=1,IS
      PH=(1.+((VIS(I)/VIS(J))**0.5)*((WM(J)/WM(I))**0.25))**2
  120 PHI(I,J)=PH/((8.*(1.+WM(I)/WM(J)))**0.5)
      VMIX=0.
      DO 160 I=1,IS
      VB=0.
      DO 140 J=1,IS
  140 VB=VB+Y(J)*PHI(I,J)                                               M
  160 VMIX=VMIX+Y(I)*VIS(I)/VB                                          M
C
C     Mixture Thermal Conductivity TCMIX (=TTRM+TINM)
C
      DO 200 I=1,IS
      TCTR(I)=2.5*VIS(I)*(1.5*R)/(WM(I)/1000.)
  200 TCINT(I)=TCC(I)-TCTR(I)
      DO 220 I=1,IS
      DO 220 J=1,IS
      AA=(1.+((TCTR(I)/TCTR(J))**0.5)*((WM(I)/WM(J))**0.25))**2
      A(I,J)=AA/((8.*(1.+WM(I)/WM(J)))**0.5)
      CC=(WM(I)-WM(J))*(WM(I)-0.142*WM(J))/((WM(I)+WM(J))**2)
  220 C(I,J)=A(I,J)*(1.+2.41*CC)
      TTRM=0.
      TINM=0.
      DO 260 I=1,IS
      TRB=0.
      TIB=0.
      DO 240 J=1,IS
      TRB=TRB+Y(J)*C(I,J)                                               M
  240 TIB=TIB+Y(J)*A(I,J)
      TTRM=TTRM+Y(I)*TCTR(I)/TRB
  260 TINM=TINM+Y(I)*TCINT(I)/TIB                                       N
      TMIX=TTRM+TINM
      RETURN
      END


