      PROGRAM EX1007
      PARAMETER (IS=2)
      DIMENSION X(IS),TC(IS),PC(IS),VC(IS),OMEG(IS),WM(IS)
      DIMENSION Y1(IS),Y2(IS),Y3(IS),Y4(IS)
C     Heptane+Undecane
      DATA TC(1),PC(1),VC(1),OMEG(1)/540.15,2.735E+6,432.,0.349/
      DATA Y1(1),Y2(1),Y3(1),Y4(1)/-5.146,6.762E-1,-3.651E-4,7.658E-8/
      DATA X(1),WM(1)/0.5,100.2034/
      DATA TC(2),PC(2),VC(2),OMEG(2)/638.85,1.955E+6,660.,0.535/
      DATA Y1(2),Y2(2),Y3(2),Y4(2)/-8.395,1.054E+0,-5.799E-4,1.237E-7/
      DATA X(2),WM(2)/0.5,156.3106/
      DATA IP,T,P/1,287.95,0.1E+6/

      CALL ELYHANL(T,P,IS,IP,X,TC,PC,VC,WM,OMEG,Y1,Y2,Y3,Y4,D,VIS,TCC)
      WMIX=0.
      DO 20 I=1,IS
   20 WMIX=WMIX+X(I)*WM(I)
      WRITE(*,1000) T,P*1.E-6,D*WMIX*1.E-3,VIS*1E+6,TCC*1.E+3
 1000 FORMAT(3X,'@ T =',F7.2,' K   &   P =',F5.2,' MPA',/,7X,
     &'DENSITY          : ',F8.1,'  KG/M3',/,7X,
     &'VISCOSITY        : ',F8.1,'  uPA.S',/,7X,
     &'TH. CONDUCTIVITY : ',F8.1,'  mW/M/K')
      END

      SUBROUTINE ELYHANL
     &(T,P,IS,IP,X,TC,PC,VC,WM,OMEG,Y1,Y2,Y3,Y4,D,VIS,TCC)
C
C     The subroutine calculates the following properties of a mixture of
C     IS components, at state IP (liquid=1, vapour=2), at temperature,
C     T in [K], and pressure, P in [PA]:
C          1) Viscosity                       (VIS)  in [PA.S]
C          2) Thermal Conductivity            (TCC)  in [W/M/K]
C          3) Density (BWR subroutine)          (D)  in [MOL/M3]
C     based on the corresponding-states scheme developed by Ely & Hanley
C        Input to the subroutine, for each component, are:
C     the molefractions X, the critical constants TC in [K], PC in
C     [Pa], VC in [cm3/mol], the molecular weights WM, the acentric
C     factors OMEG, and the four coefficients of the ideal heat-capacity
C     Y1,Y2,Y3,Y4, all for each component.
C
      COMMON /PROPCON/ VIA(7),VIB(4),VIC(9),TCA(7),TCB(4)
      DIMENSION X(IS),TC(IS),PC(IS),VC(IS),OMEG(IS),WM(IS)
      DIMENSION Y1(IS),Y2(IS),Y3(IS),Y4(IS)
      DIMENSION DC(10),H(10,10),F(10,10),WMM(10,10),TCI(10,10)
C
C     Mixture Density
C
      DO 100 I=1,IS
  100 DC(I)=1.E+6/VC(I)
      CALL BWR(T,P,IP,IS,X,TC,DC,OMEG,Y1,Y2,Y3,Y4,D,H,S,FUG)
      VMIX=1.E+6/D
C
C     Pure Components (I,I) Equivalent Fluid Reducing Parameters
C
      DO 120 I=1,IS
      TR=MIN(2.,(T/TC(I)))
      VR=MIN(2.,MAX((VMIX/VC(I)),0.5))
      THI=1.+(OMEG(I)-0.011)*(0.09057-0.86276*LOG(TR)+(0.31664-0.46568/
     &TR)*(VR-0.5))
      PHI=(1.+(OMEG(I)-0.011)*(0.39490*(VR-1.02355)-0.93281*(VR-0.75464)
     &*LOG(TR)))*(0.288*8.314*TC(I)/(PC(I)*VC(I)*1.E-6))
      F(I,I)=THI*TC(I)/190.55
      H(I,I)=PHI*VC(I)/99.2
  120 WMM(I,I)=WM(I)*1.E-3
C
C     Binary (I,J) Equivalent Fluid Reducing Parameters
C
      DO 140 I=1,IS
      DO 140 J=1,IS
      H(I,J)=(1./8.)*((H(I,I)**(1./3.))+(H(J,J)**(1./3.)))**3
      F(I,J)=(F(I,I)*F(J,J))**0.5
  140 WMM(I,J)=2.*WMM(I,I)*WMM(J,J)/(WMM(I,I)+WMM(J,J))
C
C     Mixture Equivalent Fluid Reducing Parameters and Molecular Weight
C
      HMIX=0.
      FMIX=0.
      WMIXV=0.
      WMIXT=0.
      DO 160 I=1,IS
      DO 160 J=1,IS
      HMIX=HMIX+X(I)*X(J)*H(I,J)
      FMIX=FMIX+X(I)*X(J)*F(I,J)*H(I,J)
      WMIXV=WMIXV+X(I)*X(J)*(WMM(I,J)**(+0.5))*(F(I,J)**0.5)*(H(I,J)**
     &(+4./3.))
  160 WMIXT=WMIXT+X(I)*X(J)*(WMM(I,J)**(-0.5))*(F(I,J)**0.5)*(H(I,J)**
     &(-4./3.))
      FMIX=FMIX/HMIX
      WMIXV=(WMIXV**(+2))*(FMIX**(-1))*(HMIX**(-8./3.))
      WMIXT=(WMIXT**(-2))*FMIX*(HMIX**(-8./3.))
C
C     Equivalent Temperature, T0, and Density, D0
C
      T0=T/FMIX
      D0=(16.043/VMIX)*HMIX
C
C     X correction for Viscosity & Thermal Conductivity
C
      TCMIX=0.
      VCMIX=0.
      PCMIX=0.
      OMMIX=0.
      DO 170 I=1,IS
      TCMIX=TCMIX+X(I)*TC(I)
      VCMIX=VCMIX+X(I)*VC(I)
      PCMIX=PCMIX+X(I)*PC(I)
  170 OMMIX=OMMIX+X(I)*OMEG(I)
      ZCMIX=PCMIX*VCMIX*1.E-6/(8.314*TCMIX)
      VR=MIN(2.,MAX((VMIX/VCMIX),0.5))
      DFDT=(OMMIX-0.011)*(-0.86276/T+0.46568*TCMIX*(VR-0.5)/(T**2))
      IF ((T/TCMIX).GE.2.) DFDT=0.
      DFDT=(TCMIX/190.4)*DFDT
      XVIS=((1.-1.5*T*DFDT/FMIX)*(ZCMIX/0.288))**0.5
      XTCC=((1.-1.0*T*DFDT/FMIX)*(0.288/ZCMIX))**1.5
C
C     Mixture Viscosity VIS
C
      V0=0.
      DO 180 I=1,9
  180 V0=V0+VIC(I)*(T0**(FLOAT(I-4)/3.))*1.E-7
      V1=(VIB(1)+VIB(2)*(VIB(3)-LOG(T0/VIB(4))))*D0*1.E-6
      DV=(D0**0.1)*(VIA(3)+VIA(4)/(T0**1.5))+((D0/.1617)-1.)*(D0**0.5)
     &*(VIA(5)+VIA(6)/T0+VIA(7)/(T0**2))
      DV=EXP(VIA(1)+VIA(2)/T0)*(EXP(DV)-1.)*1.E-7
      FVI=((WMIXV/0.016043)**0.5)*(FMIX**0.5)/(HMIX**(2./3.))
      VIS=FVI*(V0+V1+DV*XVIS)
C     WRITE(*,900) FVI*V0*1.E+6,FVI*V1*1.E+6,FVI*DV*1.E+6
C 900 FORMAT(4(/,7X,F10.5))
C
C     Mixture Thermal Conductivity TCTRANS : Translational part
C
      TC0=(15./4.)*(8.314/0.016043)*V0
      TC1=(TCB(1)+TCB(2)*((TCB(3)-LOG(T0/TCB(4)))**2))*D0
      DTC=(D0**0.1)*(TCA(3)+TCA(4)/(T0**1.5))+((D0/.1617)-1.)*(D0**0.5)
     &*(TCA(5)+TCA(6)/T0+TCA(7)/(T0**2))
      DTC=EXP(TCA(1)+TCA(2)/T0)*(EXP(DTC)-1.)*1.E-3
      FTC=((0.016043/WMIXT)**0.5)*(FMIX**0.5)/(HMIX**(2./3.))
      TCTRANS=FTC*(TC0+TC1+DTC)*XTCC
C
C     Mixture Thermal Conductivity TCINT : Internal part
C
      DO 220 J=1,IS
      T0=T/F(J,J)
      V0=0.
      DO 200 I=1,9
  200 V0=V0+VIC(I)*(T0**(FLOAT(I-4)/3.))*1.E-7
      FVI=((WM(J)/16.043)**0.5)*(F(J,J)**0.5)*(H(J,J)**(-2./3.))
      CV=Y1(J)+Y2(J)*T+Y3(J)*T*T+Y4(J)*(T**3)
  220 TCI(J,J)=1.32*(V0*FVI/(WM(J)*1.E-3))*(CV-2.5*8.3145)
      TCINT=0.
      DO 240 I=1,IS
      DO 240 J=1,IS
      TCI(I,J)=2.*TCI(I,I)*TCI(J,J)/(TCI(I,I)+TCI(J,J))
  240 TCINT=TCINT+X(I)*X(J)*TCI(I,J)
C
C     Total Thermal Conductivity TCC
C
      TCC=TCTRANS+TCINT
C     WRITE(*,950) FTC*TC0*1.E+3,FTC*TC1*1.E+3,FTC*DTC*1.E+3,TCINT*1.E+3
C 950 FORMAT(4(/,8X,F9.5),/)
      RETURN
      END


      BLOCK DATA
      COMMON /PROPCON/ VIA(7),VIB(4),VIC(9),TCA(7),TCB(4)
      DATA VIA/    -1.023916E+1, 1.742282E+2, 1.746055E+1,-2.847633E+3,
     & 1.336850E-1, 1.420724E+2, 5.002067E+3/
      DATA VIB/     1.696986E+0,-1.333724E-1, 1.400000E+0, 1.680000E+2/
      DATA VIC/     2.907741E+6,-3.312874E+6, 1.608102E+6,-4.331905E+5,
     & 7.062481E+4,-7.116621E+3, 4.325174E+2,-1.445911E+1, 2.037119E-1/
      DATA TCA/    -7.197708E+0, 8.567822E+1, 1.247183E+1,-9.846252E+2,
     & 3.594685E-1, 6.979841E+1,-8.728833E+2/
      DATA TCB/    -2.527629E-4, 3.343285E-4, 1.120000E+0, 1.680000E+2/
      END


      SUBROUTINE BWR
     &(T,P,IP,IS,X,TC,DC,OMEG,Y1,Y2,Y3,Y4,D,H,S,FUG)
C
C     The subroutine calculates the following thermodynamic properties
C     of mixtures, as a function of temperature, T in [K], and pressure,
C     P in [PA]:
C          1) Density                           (D)  in [MOL/M3]
C          2) Enthalpy                          (H)  in [J/MOL]
C          3) Entropy                           (S)  in [J/MOL/K]
C          4) Partial Fugacity Coefficient    (FUG)  in [-]
C     based on the B-W-R Han-Sterling EoS, shown in Table 6.7
C
C     - Enthalpy and Entropy are arbitrary assumed to be equal to zero
C       at 273.15 K and 0.1013 MPA.
C     - Perfect-gas properties are calculated from perfect-gas Heat
C       Capacities as: CP=W1+W2*T+W3*(T**2)+W4*(T**3) in [J/MOL/K]
c       where W1=Sum(Xi*Y1i) etc.
C     - Two Switches are required from the subroutine:
C       IP = Phase specification switch (Liquid=1, Vapour=2)
C       IS = Number of components in mixture (Pure fluid=1)
C     - Other data required for each component (I) are:
C       MoleFraction X (-), Critical Temperature, TC in [K], Critical
C       Density in [MOL/M3], and Acentric factor, OMEG.
C
      DIMENSION X(IS),TC(IS),DC(IS),OMEG(IS),Y1(IS)
      DIMENSION Y2(IS),Y3(IS),Y4(IS),FUG(IS)
      DIMENSION A0(10),B0(10),C0(10),D0(10),E0(10),AS(10),BS(10),CS(10),
     &DS(10),ALP(10),GAM(10),A(11),B(11)
      DATA A/0.443690,1.28438,0.356306,0.544979,0.528629,0.484011,
     &0.0705233,0.504087,0.0307452,0.0732828,0.006450/
      DATA B/0.115449,-0.920731,1.70871,-0.270896,0.349261,0.754130,
     &-0.044448,1.32245,0.179433,0.463492,-0.022143/
      DATA R,T0,P0/8.314,273.15,0.101325E+6/
C
C     Parameters A0(I),B0(I),C0(I),D0(I),E0(I),
C                AS(I),BS(I),CS(I),DS(I),     &    ALP(I),GAM(I)
C
      DO 100 I=1,IS
      A0(I)=(R*TC(I)/DC(I))*(A(2)+B(2)*OMEG(I))
      B0(I)=(1./DC(I))*(A(1)+B(1)*OMEG(I))
      C0(I)=(R*(TC(I)**3)/DC(I))*(A(3)+B(3)*OMEG(I))
      D0(I)=(R*(TC(I)**4)/DC(I))*(A(9)+B(9)*OMEG(I))
      E0(I)=(R*(TC(I)**5)/DC(I))*(A(11)+B(11)*OMEG(I)*EXP(-3.8*OMEG(I)))
      AS(I)=(R*TC(I)/(DC(I)**2))*(A(6)+B(6)*OMEG(I))
      BS(I)=(1./(DC(I)**2))*(A(5)+B(5)*OMEG(I))
      CS(I)=(R*(TC(I)**3)/(DC(I)**2))*(A(8)+B(8)*OMEG(I))
      DS(I)=(R*(TC(I)**2)/(DC(I)**2))*(A(10)+B(10)*OMEG(I))
      ALP(I)=(1./(DC(I)**3))*(A(7)+B(7)*OMEG(I))
  100 GAM(I)=(1./(DC(I)**2))*(A(4)+B(4)*OMEG(I))
C
C     Total parameters A0T,B0T,C0T,D0T,E0T,
C                      AST,BST,CST,DST,    &  ALPT,GAMT  &  WMOLT
C
      A0T=0.
      B0T=0.
      C0T=0.
      D0T=0.
      E0T=0.
      AST=0.
      BST=0.
      CST=0.
      DST=0.
      ALPT=0.
      GAMT=0.
      WMOLT=0.
      DO 200 I=1,IS
      B0T=B0T+X(I)*B0(I)
      AST=AST+X(I)*(AS(I)**(1./3.))
      BST=BST+X(I)*(BS(I)**(1./3.))
      CST=CST+X(I)*(CS(I)**(1./3.))
      DST=DST+X(I)*(DS(I)**(1./3.))
      ALPT=ALPT+X(I)*(ALP(I)**(1./3.))
      GAMT=GAMT+X(I)*(GAM(I)**(1./2.))
      DO 200 J=1,IS
      A0T=A0T+X(I)*X(J)*((A0(I)*A0(J))**0.5)
      C0T=C0T+X(I)*X(J)*((C0(I)*C0(J))**0.5)
      D0T=D0T+X(I)*X(J)*((D0(I)*D0(J))**0.5)
      E0T=E0T+X(I)*X(J)*((E0(I)*E0(J))**0.5)
  200 CONTINUE
      AST=AST**3
      BST=BST**3
      CST=CST**3
      DST=DST**3
      ALPT=ALPT**3
      GAMT=GAMT**2
C
C     Density calculation
C
      IF (IP.EQ.1) D=100000.
      IF (IP.EQ.2) D=10.
  300 F=-P+(D*R*T)
      F=F+(B0T*R*T-A0T-(C0T/(T**2))+(D0T/(T**3))-(E0T/(T**4)))*D*D
      F=F+(BST*R*T-AST-(DST/T))*(D**3)+ALPT*(AST+(DST/T))*(D**6)
      F=F+(CST/(T**2))*(D**3)*(1.+GAMT*D*D)*EXP(-GAMT*D*D)
      DF=(R*T)+2.*D*(B0T*R*T-A0T-(C0T/(T**2))+(D0T/(T**3))-(E0T/(T**4)))
      DF=DF+3.*D*D*(BST*R*T-AST-(DST/T))
      DF=DF+6.*(D**5)*ALPT*(AST+(DST/T))
      DF=DF+(CST/(T**2))*3.*(D**2)*(1.+GAMT*D*D)*EXP(-GAMT*D*D)
      DF=DF+(CST/(T**2))*(D**3)*(2.*GAMT*D)*EXP(-GAMT*D*D)
      DF=DF-2.*GAMT*D*(CST/(T**2))*(D**3)*(1.+GAMT*D*D)*EXP(-GAMT*D*D)
      H=F/DF
      IF (ABS(H/D).LE.0.000001) GOTO 400
      D=D-H
      GOTO 300
C
C     Perfect-gas Enthalpy & Entropy
C
  400 W1=0.
      W2=0.
      W3=0.
      W4=0.
      DO 500 I=1,IS
      W1=W1+X(I)*Y1(I)
      W2=W2+X(I)*Y2(I)
      W3=W3+X(I)*Y3(I)
  500 W4=W4+X(I)*Y4(I)
      PGH=W1*(T-T0)+W2*((T**2)-(T0**2))/2.+W3*((T**3)-(T0**3))/3.
     &+W4*((T**4)-(T0**4))/4.
      PGS=W1*LOG(T/T0)+W2*(T-T0)+W3*((T**2)-(T0**2))/2.
     &+W4*((T**3)-(T0**3))/3.-R*LOG(P/P0)
C
C     Enthalpy & Entropy
C
      GDD=GAMT*D*D
      H=PGH+D*((B0T*R*T)-(2.*A0T)-(4.*C0T/(T*T))+(5.*D0T/(T**3))-
     &(6.*E0T/(T**4))) + 0.5*(D**2)*(2.*BST*R*T-(3.*AST)-(4.*DST/T))
      H=H + 0.2*(D**5)*ALPT*((6.*AST)+(7.*DST/T))
      H=H + (CST/(GAMT*T*T))*(3.-(3.+0.5*GDD-GDD*GDD)*EXP(-GDD))
      SR=0.
      DO 600 I=1,IS
  600 SR=SR-X(I)*R*LOG(R*T*D*X(I)/P0)
      S=PGS+SR
      S=-D*((B0T*R)+(2.*C0T/(T**3))-(3.*D0T/(T**4))+(4.*E0T/(T**5)))
      S=S - 0.5*(D**2)*((BST*R)+(DST/(T**2)))
      S=S + 0.2*(D**5)*ALPT*DST/(T**2)
      S=S + (2.*CST/(GAMT*(T**3)))*(1.-(1.+0.5*GDD)*EXP(-GDD))
C
C     Partial Fugacities
C
      DO 800 I=1,IS
      FU=(R*T*LOG(D*R*T))+(D*R*T*(B0T+B0(I)))
      FUS=0.
      DO 700 J=1,IS
  700 FUS=FUS+X(J)*(-((A0(J)*A0(I))**0.5)-(((C0(J)*C0(I))**0.5)/(T*T))+
     &(((D0(J)*D0(I))**0.5)/(T**3))-(((E0(J)*E0(I))**0.5)/(T**4)))
      AAI=(AST*AST*AS(I))**(1./3.)
      BBI=(BST*BST*BS(I))**(1./3.)
      CCI=(CST*CST*CS(I))**(1./3.)
      DDI=(DST*DST*DS(I))**(1./3.)
      ALPI=(ALPT*ALPT*ALP(I))**(1./3.)
      GDD=GAMT*D*D
      FU=FU+(2.*D*FUS)+(3.*D*D/2.)*((BBI*R*T)-AAI-(DDI/T))
      FU=FU+(3.*ALPT*(D**5)/5.)*(AAI+(DDI/T))
      FU=FU+(3.*(D**5)/5.)*(AST+(DST/T))*ALPI
      FU=FU+(3.*CCI*D*D/(T**2))*(((1.-EXP(-GDD))/GDD)-(0.5*EXP(-GDD)))
      FU=FU-(2.*CST/(GAMT*T*T))*((GAM(I)/GAMT)**.5)*
     &(1.-(1.+GDD+0.5*GDD*GDD)*EXP(-GDD))
  800 FUG(I)=(1./P)*EXP(FU/(R*T))
      RETURN
      END
